@@ -39,9 +39,15 @@ class Catalogue:
         self.language = vals.get('language', '')
         self.source = vals.get('source', '')
         self.overview = vals.get('overview', '')
-        self.theMovieDB = vals.get('theMovieDB', '')
+        self.the_movie_db = vals.get('theMovieDB', '')
         self.rating = vals.get('rating', '')
         self.genres = vals.get('genres', [])
+        try:
+            r = int(vals.get('runtime', 0))
+        except:
+            logger.error(f"Invalid runtime {vals.get('runtime', 0)} in {self.title}")
+            r = 0
+        self.runtime = r
         self.mv_adjust = 0.0
         if 'mv' in vals:
             v = vals['mv']
@@ -83,10 +89,12 @@ class Catalogue:
             self.for_search['source'] = self.source
         if self.overview:
             self.for_search['overview'] = self.overview
-        if self.theMovieDB:
-            self.for_search['theMovieDB'] = self.theMovieDB
+        if self.the_movie_db:
+            self.for_search['theMovieDB'] = self.the_movie_db
         if self.rating:
             self.for_search['rating'] = self.rating
+        if self.runtime:
+            self.for_search['runtime'] = self.runtime
         if self.genres:
             self.for_search['genres'] = self.genres
         self.short_search = {
@@ -106,6 +114,50 @@ class Catalogue:
     def __repr__(self):
         return f"[{self.content_type}] {self.title} / {self.audio_types} / {self.year}"
 
+    @staticmethod
+    def __format_episodes(formatted, working):
+        val = ''
+        if len(formatted) > 1:
+            val += ', '
+        if len(working) == 1:
+            val += working[0]
+        else:
+            val += f"{working[0]}-{working[-1]}"
+        return val
+
+    def __format_tv_meta(self):
+        season = f"S{self.season}" if self.season else ''
+        episodes = self.episodes.split(',') if self.episodes else None
+        if episodes:
+            formatted = 'E'
+            if len(episodes) > 1:
+                working = []
+                last_value = 0
+                for ep in episodes:
+                    if len(working) == 0:
+                        working.append(ep)
+                        last_value = int(ep)
+                    else:
+                        current = int(ep)
+                        if  last_value == current - 1:
+                            working.append(ep)
+                            last_value = current
+                        else:
+                            formatted += self.__format_episodes(formatted, working)
+                            working = []
+                if len(working) > 0:
+                    formatted += self.__format_episodes(formatted, working)
+            else:
+                formatted += f"{self.episodes}"
+            return f"{season} {formatted}"
+        return season
+
+    @property
+    def formatted_title(self) -> str:
+        if self.content_type == 'TV':
+            return f"{self.title} {self.__format_tv_meta()}"
+        return self.title
+
 
 class CatalogueProvider:
 
